package skims.fu.trust.survey;

import java.util.HashMap;
import java.util.Map;

import skims.fu.survey.helper.YamlHelper;

import android.provider.CallLog.Calls;
import android.provider.ContactsContract.CommonDataKinds.Phone;

public class Call {


	private Partner contact;
	private String number;
	private long date;
	private long duration;
	private int type; // incoming, outgoing, missed
	private int numberType; // home, work, mobile, ...
	
	public static final int TYPE_UNDEFINED = -1;
	public static final int TYPE_MISSED = 0;
	public static final int TYPE_INCOMING = 1;
	public static final int TYPE_OUTGOING = 2;

	
	private String numberTypeString;
	
	private static final String DURATION = "duration";
	private static final String DATE = "relative_date";
	private static final String TYPE = "call_type";
	private static final String NUMBER_TYPE = "number_type";
	
	public static final String TYPE_MISSED_STRING = "missed";
	public static final String TYPE_INCOMING_STRING = "incoming";
	public static final String TYPE_OUTGOING_STRING = "outgoing";
	public static final String TYPE_UNDEFINED_STRING = "undefined";
	
	public Call (Map<String,Object> yamlMap) {
		this.setDate(YamlHelper.getInt(Call.DATE, yamlMap));
		this.setDuration(YamlHelper.getInt(Call.DURATION, yamlMap));
		this.setTypeString(YamlHelper.getString(Call.TYPE, yamlMap));
		this.setNumberTypeString(YamlHelper.getString(Call.NUMBER_TYPE, yamlMap));
	}
	
	public static boolean checkYaml(Map<String, Object> yamlMap) {
		if (!yamlMap.containsKey(Call.DURATION)) return false;
		if (!yamlMap.containsKey(Call.DATE)) return false;
		if (!yamlMap.containsKey(Call.TYPE)) return false;
		if (!yamlMap.containsKey(Call.NUMBER_TYPE)) return false;
		return true;
	}
	
	
	public Call() {
		this.setDate(-1);
		this.setDuration(-1);
		this.setType(-1);
		this.setNumberType(-1);
	}

	public Partner getPartner() {
		return contact;
	}
	public void setPartner(Partner contact) {
		this.contact = contact;
	}


	public String getNumber() {
		return number;
	}
	public void setNumber(String number) {
		this.number = number;
	}
	public long getDate() {
		return date;
	}


	public void setDate(long date) {
		this.date = date;
	}
	public long getDuration() {
		return duration;
	}
	public void setDuration(long duration) {
		this.duration = duration;
	}
	public int getType() {
		return type;
	}
	public void setTypeString(String typeString) {
		if (typeString == TYPE_UNDEFINED_STRING)  this.type = Call.TYPE_UNDEFINED;
		if (typeString == TYPE_MISSED_STRING)  this.type = Call.TYPE_MISSED;
		if (typeString == TYPE_INCOMING_STRING)  this.type = Call.TYPE_INCOMING;
		if (typeString == TYPE_OUTGOING_STRING)  this.type = Call.TYPE_OUTGOING;
	}
	
	public String getTypeString() {
		if (type == TYPE_UNDEFINED) return TYPE_UNDEFINED_STRING;
		return (type == TYPE_MISSED ? TYPE_MISSED_STRING : (type == TYPE_INCOMING ? TYPE_INCOMING_STRING : TYPE_OUTGOING_STRING));
	}
	public void setType(int type) {
		int thisType = 0;
		switch(type) {
		case Calls.MISSED_TYPE:
			thisType = Call.TYPE_MISSED;
			break;
		case Calls.INCOMING_TYPE:
			thisType = Call.TYPE_INCOMING;
			break;
		case Calls.OUTGOING_TYPE:
			thisType = Call.TYPE_OUTGOING;
			break;
		}
		this.type = thisType;
	}
	

	public Map<String, Object> toYaml() {
		HashMap<String,Object> map = new HashMap<String,Object>();
		
		map.put(Call.DURATION, this.getDuration());
		map.put(Call.DATE, this.getDate());
		map.put(Call.TYPE, this.getTypeString());
		map.put(Call.NUMBER_TYPE, this.getNumberTypeString());
		
		return map;
	}

	public void setNumberType(int type) {
		this.numberType = type;
	}
	public int getNumberType() {
		return this.numberType;
	}
	
	public String getNumberTypeString() {
		if (this.numberType == -2) return this.numberTypeString;
		return Call.getNumberTypeString(this.numberType);
	}

	public void setNumberTypeString(String numberTypeString) {
		this.numberTypeString = numberTypeString;
		this.numberType = -2;
	}
	
	/**
	 * Static function to map the phone_number_type to a String
	 * @param numberType The numberType as defined by ContactsContract.CommonDataKinds.Phone
	 * @return A String representing the value
	 */
	public static String getNumberTypeString(int numberType) {
		String s = "";
		switch(numberType) {
		case Phone.TYPE_HOME:
			s = "home";
			break;
		case Phone.TYPE_MOBILE:
			s = "mobile";
			break;
		case Phone.TYPE_WORK:
			s = "work";
			break;
		case Phone.TYPE_FAX_WORK:
			s = "fax_work";
			break;
		case Phone.TYPE_FAX_HOME:
			s = "fax_home";
			break;
		case Phone.TYPE_OTHER:
			s = "other";
			break;
		case Phone.TYPE_CALLBACK:
			s = "callback";
			break;
		case Phone.TYPE_CAR:
			s = "car";
			break;
		case Phone.TYPE_COMPANY_MAIN:
			s = "company_main";
			break;
		case Phone.TYPE_ISDN:
			s = "isdn";
			break;
		case Phone.TYPE_MAIN:
			s = "main";
			break;
		case Phone.TYPE_OTHER_FAX:
			s = "other fax";
			break;
		case Phone.TYPE_RADIO:
			s = "radio";
			break;
		case Phone.TYPE_TELEX:
			s = "telex";
			break;
		case Phone.TYPE_TTY_TDD:
			s = "tty_tdd";
			break;
		case Phone.TYPE_WORK_MOBILE:
			s = "work_mobile";
			break;
		case Phone.TYPE_WORK_PAGER:
			s = "work_pager";
			break;
		case Phone.TYPE_ASSISTANT:
			s = "assistant";
			break;
		case Phone.TYPE_MMS:
			s = "mms";
			break;
		}
		return s;
	}

}
