package skims.fu.trust.survey;

import java.util.HashMap;
import java.util.Map;

import skims.fu.survey.helper.YamlHelper;

public class Message {

	private String address; // the phone number of the comm. partner
	private long date;
	private int type;
	private int length; // number of characters
	private int numberType;

	public static final int TYPE_UNDEFINED = 0;
	public static final int TYPE_MISSED = 0; // possible?
	public static final int TYPE_INCOMING = 1;
	public static final int TYPE_OUTGOING = 2;
	public static final int TYPE_DRAFT = 3;

	private final static String LENGTH = "length";
	private final static String DATE = "relative_date";
	private final static String TYPE = "message_type";
	private final static String NUMBER_TYPE = "number_type";

	public static final String TYPE_MISSED_STRING = "missed";
	public static final String TYPE_INCOMING_STRING = "incoming";
	public static final String TYPE_OUTGOING_STRING = "outgoing";
	public static final String TYPE_UNDEFINED_STRING = "undefined";

	private String numberTypeString;

	public Message (Map<String,Object> yamlMap) {
		this.setLength(YamlHelper.getInt(Message.LENGTH, yamlMap));
		this.setDate(YamlHelper.getInt(Message.DATE, yamlMap));
		this.setTypeString(YamlHelper.getString(Message.TYPE, yamlMap));
		this.setNumberTypeString(YamlHelper.getString(Message.NUMBER_TYPE, yamlMap));

	}

	public static boolean checkYaml(Map<String, Object> yamlMap) {
		if (!yamlMap.containsKey(Message.LENGTH)) return false;
		if (!yamlMap.containsKey(Message.DATE)) return false;
		if (!yamlMap.containsKey(Message.TYPE)) return false;
		if (!yamlMap.containsKey(Message.NUMBER_TYPE)) return false;
		return true;
	}


	public Message() {
		this.setDate(-1);
		this.setLength(-1);
		this.setNumberType(-1);
		this.setType(-1);
	}


	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public long getDate() {
		return date;
	}

	public void setDate(long date) {
		this.date = date;
	}


	public int getType() {
		return type;
	}

	public void setType(int type) {
		int thisType = 0;
		switch(type) {
		case 1:
			thisType = TYPE_INCOMING;
			break;
		case 2:
			thisType = TYPE_OUTGOING;
			break;
		}
		this.type = thisType;	
	}



	public String getTypeString() {
		return (type == TYPE_INCOMING ? "incoming" : "outgoing");
	}

	public int getLength() {
		return this.length;
	}

	public void setLength(int length) {
		this.length = length;
	}

	public void setTypeString(String typeString) {
		if (typeString == TYPE_UNDEFINED_STRING)  this.type = Call.TYPE_UNDEFINED;
		if (typeString == TYPE_MISSED_STRING)  this.type = Call.TYPE_MISSED;
		if (typeString == TYPE_INCOMING_STRING)  this.type = Call.TYPE_INCOMING;
		if (typeString == TYPE_OUTGOING_STRING)  this.type = Call.TYPE_OUTGOING;
	}


	public Map<String, Object> toYaml() {
		HashMap<String,Object> map = new HashMap<String,Object>();

		map.put(Message.LENGTH, this.getLength());
		map.put(Message.DATE, this.getDate());
		map.put(Message.TYPE, this.getTypeString());
		map.put(Message.NUMBER_TYPE, this.getNumberTypeString());

		return map;
	}

	public void setNumberType(int type) {
		this.numberType = type;
	}
	public int getNumberType() {
		return this.numberType;
	}
	public String getNumberTypeString() {
		if (this.numberType == -2) return this.numberTypeString;
		return Call.getNumberTypeString(this.numberType);
	}

	public void setNumberTypeString(String numberTypeString) {
		this.numberTypeString = numberTypeString;
		this.numberType = -2;
	}


}
