package skims.fu.trust.survey;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import skims.fu.survey.helper.YamlHelper;


public class Participant {
	
	private List<Partner> partners;
 
	private String versionString;
	private int numberPartnersWithoutName;
	private int numberPartnersTotal;
	private int numberContactsTotal;
	private int numberCallsTotal;
	private int numberMessagesTotal;
	
	private static final String GENERAL_SECTION = "general";
	private static final String LIST_OF_PARTNERS = "partners";

	private static final String VERSION = "version";
	private static final String NUMBER_PARTNERS_WITHOUT_NAME = "number_partners_without_name";
	private static final String NUMBER_PARTNERS_TOTAL = "number_partners_total";
	private static final String NUMBER_CONTACTS_TOTAL = "number_contacts_total";
	private static final String NUMBER_CALLS_TOTAL = "number_calls_total";
	private static final String NUMBER_MESSAGES_TOTAL = "number_messages_total";
	
	public Participant() {
		this.partners = new LinkedList<Partner>();
		
		this.setVersionString(null);
		this.setNumberPartnersWithoutName(-1);
		this.setNumberPartnersTotal(-1);
		this.setNumberContactsTotal(-1);
		this.setNumberCallsTotal(-1);
		this.setNumberMessagesTotal(-1);		
	}
	
	public Participant(Map<String, Object> yamlMap) {

		// extract the general information
		Map<String, Object> generalMap = YamlHelper.getMap(Participant.GENERAL_SECTION, yamlMap);
		
		this.setVersionString(YamlHelper.getString(Participant.VERSION, generalMap));
		this.setNumberPartnersWithoutName(YamlHelper.getInt(Participant.NUMBER_PARTNERS_WITHOUT_NAME, generalMap));
		this.setNumberPartnersTotal(YamlHelper.getInt(Participant.NUMBER_PARTNERS_TOTAL, generalMap));
		this.setNumberContactsTotal(YamlHelper.getInt(Participant.NUMBER_CONTACTS_TOTAL, generalMap));
		this.setNumberCallsTotal(YamlHelper.getInt(Participant.NUMBER_CALLS_TOTAL, generalMap));
		this.setNumberMessagesTotal(YamlHelper.getInt(Participant.NUMBER_MESSAGES_TOTAL, generalMap));
		this.partners = new LinkedList<Partner>();
		
		// extract the list of partners
        List<Map<String,Object>> partnersList = YamlHelper.getListOfMaps(Participant.LIST_OF_PARTNERS, yamlMap);
    	
       	for (Map<String,Object> partnerMap : partnersList) {
       		if (Partner.checkYaml(partnerMap)) {
       			this.partners.add(new Partner(partnerMap));
       		}
       	}
	}
	
	public static boolean checkYaml(Map<String, Object> yamlMap) {
		if (!yamlMap.containsKey(Participant.GENERAL_SECTION)) return false;
		if (!yamlMap.containsKey(Participant.LIST_OF_PARTNERS)) return false;
		return true;
	}
	
	public void addPartner(Partner partner) {
		this.partners.add(partner);
	}
	
	public void addPartners(Collection<Partner> partners) {
		this.partners.addAll(partners);
	}
	
	public List<Partner> getPartners() {
		return this.partners;
	}
	
	public Map<String, Object> toYaml() {
		// create root section for this survey-participant
		Map<String, Object> rootMap = new HashMap<String, Object>();
		
		// create general section
		Map<String, Object> generalMap = new HashMap<String,Object>();
				
		generalMap.put(Participant.VERSION, this.versionString);
		generalMap.put(Participant.NUMBER_PARTNERS_WITHOUT_NAME, this.numberPartnersWithoutName);
		generalMap.put(Participant.NUMBER_PARTNERS_TOTAL, this.numberPartnersTotal);
		generalMap.put(Participant.NUMBER_CONTACTS_TOTAL, this.numberContactsTotal);
		generalMap.put(Participant.NUMBER_CALLS_TOTAL, this.numberCallsTotal);
		generalMap.put(Participant.NUMBER_MESSAGES_TOTAL, this.numberMessagesTotal);
		
		rootMap.put(Participant.GENERAL_SECTION, generalMap);
		
		// create partners section
		List<Map<String,Object>> partnersList = new LinkedList<Map<String,Object>>();
		
		if (this.partners != null)
			for (Partner partner : this.partners) {
				partnersList.add(partner.toYaml());
			}
		
		rootMap.put(Participant.LIST_OF_PARTNERS, partnersList);
				
		return rootMap;
	}

	public String getVersionString() {
		return versionString;
	}

	public void setVersionString(String versionString) {
		this.versionString = versionString;
	}
	public int getNumberPartnersTotal() {
		return numberPartnersTotal;
	}

	public void setNumberPartnersTotal(int numberPartnersTotal) {
		this.numberPartnersTotal = numberPartnersTotal;
	}

	public int getNumberContactsTotal() {
		return numberContactsTotal;
	}

	public void setNumberContactsTotal(int numberContactsTotal) {
		this.numberContactsTotal = numberContactsTotal;
	}

	public int getNumberCallsTotal() {
		return numberCallsTotal;
	}

	public void setNumberCallsTotal(int numberCallsTotal) {
		this.numberCallsTotal = numberCallsTotal;
	}

	public int getNumberMessagesTotal() {
		return numberMessagesTotal;
	}

	public void setNumberMessagesTotal(int numberMessagesTotal) {
		this.numberMessagesTotal = numberMessagesTotal;
	}

	public int getNumberPartnersWithoutName() {
		return numberPartnersWithoutName;
	}
	
	public void setNumberPartnersWithoutName(int numberPartnersWithoutName) {
		this.numberPartnersWithoutName = numberPartnersWithoutName;
	}

}
