package skims.fu.trust.survey;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import skims.fu.survey.helper.YamlHelper;

public class Partner {

	// how many seconds of a phone call is a symbol in a message worth
	public static final int MSG_LENGTH_TO_PHONE_SECOND_FACTOR = 1; 	

	private String lookupKey; // the lookup key of the content provider
	private String displayName;
	
	private boolean human;
	
	private boolean inContactList;
	private int indexInRatingList;
	
	private int ratingCloseness;
	private int ratingTrustInformation;
	private int ratingTrustBest;
	
	// needed to quickly compare entries during data processing 
	private Map<String,Integer> numbers;
	
	private boolean starred; // is this partner in the favorite group of the contact list?
	
	private int numCallsIn;
	private int numCallsOut;
	private int numCallsMissed;
	private int durCallsIn;
	private int durCallsOut;
	
	private int numMessagesIn;
	private int numMessagesOut;
	private int lenMessagesIn;
	private int lenMessagesOut;

	
	private List<Call> calls;
	private List<Message> messages;
	
	private static final String SECTION_CALLS = "calls";
	private static final String SECTION_MESSAGES = "messages";
	
	private static final String RATING_CLOSENESS = "rating_closeness";
	private static final String RATING_TRUST_INFORMATION = "rating_trust_information";
	private static final String RATING_TRUST_BEST = "rating_trust_best";
	private static final String STARRED = "starred";
	private static final String HUMAN = "human";
	private static final String INDEX_IN_RATING_LIST = "index_in_rating_list";
	
	public Partner(String lookupKey) {
		this.lookupKey = lookupKey;
		this.calls = new LinkedList<Call>();
		this.messages = new LinkedList<Message>();
		this.setRatingCloseness(-1);
		this.setRatingTrustInformation(-1);
		this.setRatingTrustBest(-1);
		this.setStarred(0);
		this.setIndexInRatingList(-1);
		this.human = true;
		this.numbers = new HashMap<String, Integer>();

		this.numCallsIn = 0;
		this.numCallsOut = 0;
		this.numCallsMissed = 0;
		this.durCallsIn = 0;
		this.durCallsOut = 0;
		
		this.numMessagesIn = 0;
		this.numMessagesOut = 0;
		this.lenMessagesIn = 0;
		this.lenMessagesOut = 0;
	
	}

	public Partner(Map<String, Object> yamlMap) {
		this.setRatingCloseness(YamlHelper.getInt(Partner.RATING_CLOSENESS, yamlMap));
		this.setRatingTrustInformation(YamlHelper.getInt(Partner.RATING_CLOSENESS, yamlMap));
		this.setRatingTrustBest(YamlHelper.getInt(Partner.RATING_CLOSENESS, yamlMap));
		this.setStarred(YamlHelper.getInt(Partner.RATING_CLOSENESS, yamlMap));
		this.setIndexInRatingList(YamlHelper.getInt(Partner.RATING_CLOSENESS, yamlMap));
		this.human = (YamlHelper.getBoolean(Partner.HUMAN, yamlMap));
		this.calls = new LinkedList<Call>();
		this.messages = new LinkedList<Message>();

		List<Map<String, Object>> callsList = YamlHelper.getListOfMaps(Partner.SECTION_CALLS, yamlMap);
		for (Map<String, Object> callMap : callsList) {
			if (Call.checkYaml(callMap)) {
				this.addCall(new Call(callMap));
			}
		}
		
		List<Map<String, Object>> messagesList = YamlHelper.getListOfMaps(Partner.SECTION_MESSAGES, yamlMap);
		for (Map<String, Object> messageMap : messagesList) {
			if (Message.checkYaml(messageMap)) {
				this.addMessage(new Message(messageMap));
			}
		}
	
	}
	
	public static boolean checkYaml(Map<String, Object> yamlMap) {
		if (!yamlMap.containsKey(Partner.SECTION_CALLS)) return false;
		if (!yamlMap.containsKey(Partner.SECTION_MESSAGES)) return false;
		if (!yamlMap.containsKey(Partner.RATING_CLOSENESS)) return false;
		if (!yamlMap.containsKey(Partner.RATING_TRUST_INFORMATION)) return false;
		if (!yamlMap.containsKey(Partner.RATING_TRUST_BEST)) return false;
		if (!yamlMap.containsKey(Partner.STARRED)) return false;
		if (!yamlMap.containsKey(Partner.HUMAN)) return false;
		if (!yamlMap.containsKey(Partner.INDEX_IN_RATING_LIST)) return false;
		return true;
	}
	
	public void setNotHuman() {
		this.human = false;
	}
	
	public boolean isHuman() {
		return this.human;
	}
	
	public void addCall(Call newCall) {
		if (newCall.getType() == Call.TYPE_INCOMING) {
			this.numCallsIn++;
			this.durCallsIn += newCall.getDuration();
		} else if (newCall.getType() == Call.TYPE_OUTGOING) {
			this.numCallsOut++;
			this.durCallsOut += newCall.getDuration();
		} else if (newCall.getType() == Call.TYPE_MISSED) {
			this.numCallsMissed++;
		}
		calls.add(newCall);
	}
	

	public void addMessage(Message newMessage) {
		if (newMessage.getType() == Message.TYPE_INCOMING) {
			this.numMessagesIn ++;
			this.lenMessagesIn += newMessage.getLength();
		} else if (newMessage.getType() == Message.TYPE_OUTGOING) {
			this.numMessagesOut ++;
			this.lenMessagesOut += newMessage.getLength();
		}
		messages.add(newMessage);
	}
	
	public void setIndexInRatingList(int idx) {
		this.indexInRatingList = idx;
	}
	
	public int getIndexInRatingList() {
		return this.indexInRatingList;
	}
	
	public boolean inContactList() {
		return this.inContactList;
	}
	
	public void setInContactList(boolean in) {
		this.inContactList = in;
	}
	
	public void setRatingCloseness(int closeness) {
		this.ratingCloseness = closeness;
	}
	
	public int getRatingCloseness() {
		return this.ratingCloseness;
	}
	
	public void setRatingTrustInformation(int trust) {
		this.ratingTrustInformation = trust;
	}
	
	public int getRatingTrustInformation() {
		return this.ratingTrustInformation;
	}
	
	public void setRatingTrustBest(int trust) {
		this.ratingTrustBest = trust;
	}
	
	public int getRatingTrustBest() {
		return this.ratingTrustBest;
	}
	public boolean hasNumber(String number) {
		return this.numbers.containsKey(number);
	}
	public void addNumber(String number, int phoneNumberType) {
		this.numbers.put(number, phoneNumberType);
	}
	public int getNumberType(String number) {
		return this.numbers.get(number);
	}

	
	public int getNumMessages() {
		return this.numMessagesIn + this.numMessagesOut;
	}
	public int getNumCalls() {
		return this.numCallsIn+this.numCallsOut+this.numCallsMissed;
	}
	public int getNumInteractions() {
		return this.getNumMessages() + this.getNumCalls();
	}
	public int getMessageLength() {
		return this.lenMessagesIn+this.lenMessagesOut;
	}
	public int getCallDuration() {
		return this.durCallsIn+this.durCallsOut;
	}
	
	public List<Call> getCalls() {
		return this.calls;
	}
	
	public List<Message> getMessages() {
		return this.messages;
	}
	
	public String getLookupKey() {
		return lookupKey;
	}
	public boolean equals(String lookupKey) {
		return this.lookupKey.equals(lookupKey);
	}
	
	public void setDisplayName(String name) {
		this.displayName = name;
	}
	
	public String getDisplayName() {
		return this.displayName;
	}
	public boolean getStarred() {
		return starred;
	}
	public void setStarred(int star) {
		this.starred = star == 1;
	}

	
	public Map<String, Object> toYaml() {
		HashMap<String,Object> map = new HashMap<String,Object>();
		
		map.put(Partner.HUMAN, this.isHuman());
		map.put(Partner.INDEX_IN_RATING_LIST, this.indexInRatingList);
		map.put(Partner.RATING_CLOSENESS, this.getRatingCloseness());
		map.put(Partner.RATING_TRUST_INFORMATION, this.getRatingTrustInformation());
		map.put(Partner.RATING_TRUST_BEST, this.getRatingTrustBest());
		map.put(Partner.STARRED, this.getStarred());
		
	
		
		// create entries for the calls
		LinkedList<Map<String, Object>> callList = new LinkedList<Map<String,Object>>();
		
		if (this.calls != null)
			for (Call call : this.calls) {
				callList.add(call.toYaml());
			}
		map.put(Partner.SECTION_CALLS, callList);

		// create entries for the message
		LinkedList<Map<String, Object>> messageList = new LinkedList<Map<String,Object>>();
		
		if (this.messages != null)
			for (Message message : this.messages) {
				messageList.add(message.toYaml());
			}
		map.put(Partner.SECTION_MESSAGES, messageList);
		
		return map;
	}

}
