package skims.fu.trust.survey;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.yaml.snakeyaml.DumperOptions;
import org.yaml.snakeyaml.Yaml;

import skims.fu.R;
import skims.fu.trust.survey.Message;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageInfo;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.CallLog.Calls;
import android.provider.ContactsContract;
import android.provider.ContactsContract.PhoneLookup;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.RatingBar;
import android.widget.TextView;

public class SkimsTrustSurvey extends Activity implements OnClickListener {


	private final String 	RESULT_EMAIL		= "skims@inf.fu-berlin.de";
	private final int 		NUMBER_RATINGS 		= 20;
	

	// Number of contacts that are prepared and ready to substitute a "not a person" entry
	private final int		NUMBER_BACKUP_RATINGS = 10;
	
	// Should the participant be allowed to cancel early?
	private final boolean	ALLOW_EARLY_CANCEL	= false;
	
	// Set to true if you only want to analyze interactions with partners that
	// are in the contact list (thus, have a name, ...)
	private final boolean	USE_ONLY_CONTACT_LIST_ENTRIES = true;
	
	// Out of how many "pots" will the rating partners be drawn
	private final int		NUMBER_SELECTION_CLASSES = 3;
	
	// Holds all partners the are recognized (there has been contact). 
	// The key is the lookup key as provided by ANDROID (for interactions not in the contact list, it's a prefix + the number)
	private Map<String, Partner> partners;

	// An ordered List of Partners that will be shown for rating
	private List<Partner> ratingPartners;

	private List<Partner>[] ratingPartnersBackup;
	
	// The random offset to be added to the date to anonymize it
	private int randomDateOffset;
	
	// What is the index of the actual partner being rated
	private int ratingPartnerIdx;

	private int numCalls;
	private int numMessages;
	private int numContacts;
	
	private Dialog startScreen;

	/**
	 * Definition of SMS Content Provider Column Names (no official documentation)
	 */
	private final String SMS_COL_ADDRESS = "address";
	private final String SMS_COL_DATE = "date";
	private final String SMS_COL_TYPE = "type";
	private final String SMS_COL_BODY = "body";

	public final String LOOKUP_KEY_CREATION_PREFIX = "NEW_";

	
	public void onDestroy() {
		super.onDestroy();

		if (this.startScreen.isShowing()) this.startScreen.dismiss();
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		// START Main Screen Setup
			setContentView(R.layout.rating);
	
			// set up the action for the button on the main screen
			Button btn_next = (Button)findViewById(R.id.btn_next);
			btn_next.setOnClickListener(this);
	
			TextView btn_cancel = (TextView)findViewById(R.id.btn_cancel);
			if (this.ALLOW_EARLY_CANCEL) {
				btn_cancel.setOnClickListener(this);
			} else {
				btn_cancel.setVisibility(View.INVISIBLE);
			}
			
			TextView tv_not_a_person = (TextView)findViewById(R.id.tv_not_a_person);
			tv_not_a_person.setOnClickListener(this);
			
			RatingBar rCloseness = (RatingBar) findViewById(R.id.rating_closeness);
			rCloseness.setStepSize(1);
			rCloseness.setMax(5);
			rCloseness.setRating(1);
	
			RatingBar rTrustInformation = (RatingBar) findViewById(R.id.rating_trust_information);
			rTrustInformation.setStepSize(1);
			rTrustInformation.setMax(5);
			rTrustInformation.setRating(1);

			RatingBar rTrustBest = (RatingBar) findViewById(R.id.rating_trust_best);
			rTrustBest.setStepSize(1);
			rTrustBest.setMax(5);
			rTrustBest.setRating(1);
		// END Main Screen Setup
			
		// START Start Screen Setup
			// setup and show the startup screen before analyzing the call logs		
			this.startScreen = new Dialog(SkimsTrustSurvey.this);
			this.startScreen.setContentView(R.layout.start_screen);
			this.startScreen.setTitle(R.string.start_title);
			this.startScreen.setCancelable(false);
			
			//set up the start-screen button
			Button btn_start = (Button) this.startScreen.findViewById(R.id.btn_start);
			btn_start.setOnClickListener(this);
	
			// Button is invisible at first and pops up as soon as the logs are analyzed
			btn_start.setVisibility(View.INVISIBLE);
	
			// show the start screen
			startScreen.show();
		// END Start Screen Setup
		
		// set global variables
		this.ratingPartnerIdx = 0;
		// create a random int to be added to the date
		Random randomGen = new Random();
		this.randomDateOffset = randomGen.nextInt(1000000);
		
		// Analyze the logs in an asynchronous task
		new LogAnalyzerTask().execute();
	}


	public void onClick (View view) {
		switch (view.getId()) {
		case R.id.btn_start:
			startScreen.dismiss();
			// check if there are contacts to rate, if not, show a quick dialog and finish()
			if (this.ratingPartners.isEmpty()) {
				AlertDialog.Builder builder = new AlertDialog.Builder(this);

				builder.setMessage(getString(R.string.no_partners_dialog))
					   .setTitle(R.string.no_partners_title)
				       .setCancelable(false)
				       .setNegativeButton(getString(R.string.button_close), new DialogInterface.OnClickListener() {
				    	   public void onClick(DialogInterface dialog, int id) {
				    		   dialog.cancel();
				    		   SkimsTrustSurvey.this.finish();
				    	   }
				       });
				AlertDialog noPartnersDialog = builder.create();
				noPartnersDialog.show();
				break;
			}
			// put the first partner to rate on the screen
			putPartnerOnScreen();
			break;
		case R.id.btn_next:
			saveRating();
			prepareNextPartner();
			break;
		case R.id.btn_cancel:
			// Show the "really exit?" dialog
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setMessage(getString(R.string.cancel_early_dialog))
				   .setCancelable(true)
			       .setPositiveButton(getString(R.string.button_confirm_cancel), new DialogInterface.OnClickListener() {
			           public void onClick(DialogInterface dialog, int id) {
			        		SkimsTrustSurvey.this.saveRating();
			        		SkimsTrustSurvey.this.sendData(prepareData());
			        		SkimsTrustSurvey.this.finish();
			           }
			       })
			       .setNegativeButton(getString(R.string.button_keep_rating), new DialogInterface.OnClickListener() {
			           public void onClick(DialogInterface dialog, int id) {
			                dialog.cancel();
			           }
			       });
			AlertDialog cancelDialog = builder.create();
			cancelDialog.show();
			break;
		case R.id.tv_not_a_person:
			
			// fix for fast pressing of this is not a person button while it is preparing the data for the email 
			if (ratingPartnerIdx >= this.ratingPartners.size()) break;
			
			// set the partner "not-human" property and put it on the "not-human" list
			this.ratingPartners.get(this.ratingPartnerIdx).setNotHuman();
			
			// remove this partner from the list and put in a new one as substitute
			this.ratingPartners.remove(this.ratingPartnerIdx);
			int backupIndex = this.ratingPartnerIdx % this.NUMBER_SELECTION_CLASSES;
			if (!this.ratingPartnersBackup[backupIndex].isEmpty()) {
				Partner backupPartner = this.ratingPartnersBackup[backupIndex].get(0);
				backupPartner.setIndexInRatingList(this.ratingPartnerIdx);
				this.ratingPartners.add(backupPartner);
				this.ratingPartnersBackup[backupIndex].remove(0);
			}
			this.ratingPartnerIdx--;
			prepareNextPartner();
			break;
		}
	}

	private void prepareNextPartner() {
		this.ratingPartnerIdx++;
		if (this.ratingPartnerIdx >= this.ratingPartners.size()) {
			//end the rating and prepare the data for sending out
			sendData(prepareData());
			finish();
		} else {
			putPartnerOnScreen();
		}
	}

	private StringWriter prepareData() {
		return prepareYAMLData();
	}
	
	/**
	 * Prepares the data in YAML to be send out
	 */
	private StringWriter prepareYAMLData() {
		
		// find out how many partners are in the contact list - as opposed to being "just numbers"
		int numPartners = 0;
		if (this.USE_ONLY_CONTACT_LIST_ENTRIES) {
			numPartners = this.partners.size();
		} else {
			for (Partner partner : this.partners.values()) {
				if (partner.inContactList()) numPartners++;
			}
		}
		
		String versionString = "-1";
		
		try {
			PackageInfo pinfo = getPackageManager().getPackageInfo(getPackageName(), 0);
			versionString = pinfo.versionName+ "." + this.NUMBER_RATINGS + "." + pinfo.versionCode;
		} catch (Exception e) {
			versionString = "1.0"+this.NUMBER_RATINGS;
		}
		
		// create a Participant object that can easily be printed out as yaml
		Participant participant = new Participant();
		
		participant.setVersionString(versionString);
		participant.setNumberPartnersWithoutName(this.partners.size() - numPartners);
		participant.setNumberPartnersTotal(numPartners);
		participant.setNumberContactsTotal(this.numContacts);
		participant.setNumberCallsTotal(this.numCalls);
		participant.setNumberMessagesTotal(this.numMessages);
		
		participant.addPartners(this.partners.values());
		
		// prepare the YAML object
		DumperOptions options = new DumperOptions();
		options.setDefaultFlowStyle(DumperOptions.FlowStyle.BLOCK);
			
		StringWriter writer = new StringWriter();
		Yaml yaml = new Yaml(options);
				
		// put the contents of <yamlMap> in YAML format to <writer>
		yaml.dump(participant.toYaml(), writer);
		
		return writer;
	}

	/**
	 * Is sending out the data (at first by creating an email)
	 * @param writer A String writer containing the Data to be send out
	 */
	private void sendData(StringWriter writer) {
		// sending the data by email
		final Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);
		emailIntent.setType("plain/text");
		emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, new String[]{ RESULT_EMAIL });
		emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, getString(R.string.email_subject));
		emailIntent.putExtra(android.content.Intent.EXTRA_TEXT, writer.toString());
		
		this.startActivity(Intent.createChooser(emailIntent, getString(R.string.email_chooser)));
	}


	/**
	 * Put the next partner's info on the rating screen
	 */
	private void putPartnerOnScreen() {
		RatingBar rCloseness = (RatingBar) findViewById(R.id.rating_closeness);
		RatingBar rTrustInformation = (RatingBar) findViewById(R.id.rating_trust_information);
		RatingBar rTrustBest = (RatingBar) findViewById(R.id.rating_trust_best);
		
		rCloseness.setRating(1);
		rTrustInformation.setRating(1);
		rTrustBest.setRating(1);
		
		// put the partner's info on the screen
		TextView partnerName = (TextView) findViewById(R.id.rating_partner_name);
		partnerName.setText(this.ratingPartners.get(this.ratingPartnerIdx).getDisplayName());

		// check if this is the last rating to be done
		if (this.ratingPartnerIdx == this.ratingPartners.size() - 1) {
			// get the button and change its caption
			Button btn_next = (Button) findViewById(R.id.btn_next);
			btn_next.setText(R.string.button_finish);
			
			TextView btn_cancel = (TextView) findViewById(R.id.btn_cancel);
			btn_cancel.setVisibility(View.INVISIBLE);
		}

		TextView pageNumber = (TextView) findViewById(R.id.page_number);
		pageNumber.setText((this.ratingPartnerIdx+1) + "/" + this.ratingPartners.size());
	}

	/**
	 * Save the current partners info from the screen in the partner object
	 */
	private void saveRating() {
		RatingBar rCloseness = (RatingBar) findViewById(R.id.rating_closeness);
		RatingBar rTrustInformation = (RatingBar) findViewById(R.id.rating_trust_information);
		RatingBar rTrustBest = (RatingBar) findViewById(R.id.rating_trust_best);
		
		int ratingCloseness = (int)rCloseness.getRating();
		int ratingTrustInformation = (int)rTrustInformation.getRating();
		int ratingTrustBest = (int)rTrustBest.getRating();
		
		this.ratingPartners.get(this.ratingPartnerIdx).setRatingCloseness(ratingCloseness);
		this.ratingPartners.get(this.ratingPartnerIdx).setRatingTrustInformation(ratingTrustInformation);
		this.ratingPartners.get(this.ratingPartnerIdx).setRatingTrustBest(ratingTrustBest);
	}

	/**
	 * Out of the list of partners choose the ones that will be used for rating
	 * @param partners the list of partners that have interacted with the user
	 * @return
	 */
	private void createRatingPartners() {
		// create rankings and wheightenings 
		LinkedList<Partner> ratingPartners = new LinkedList<Partner>();
	
		// create ratings concerning...
		List<Partner> numInteractionsOrder = Sort.sortPartnerBy(this.partners.values(), Sort.BY_NUMBER_INTERACTIONS);

		// compose the list of rating partners

		int index;
		Random randomGen = new Random();
		Partner nextPartner;

		// Scheme: HIGH INTERACTION - LOW INTERACTION - RANDOM - HIGH INTERACTION - ...
		for (int i=0; i < this.NUMBER_RATINGS; i++) {

			if (numInteractionsOrder.size() == 0) break;
			index = getIndex(i, numInteractionsOrder.size(), randomGen);
			nextPartner =  numInteractionsOrder.get(index);

			while (ratingPartners.contains(nextPartner)) {
				numInteractionsOrder.remove(index);
				if (numInteractionsOrder.size() == 0) break;
				index = getIndex(i, numInteractionsOrder.size(), randomGen);
				nextPartner = numInteractionsOrder.get(index);
			}

			nextPartner.setIndexInRatingList(i);
			ratingPartners.add(nextPartner);
			numInteractionsOrder.remove(index);
		}

		this.ratingPartners = ratingPartners;
		
		// create backup rating partners
		
		ArrayList<Partner>[] backupPartners = (ArrayList<Partner>[]) new ArrayList[3];
		for (int i=0; i < this.NUMBER_SELECTION_CLASSES; i++) {
			backupPartners[i] = new ArrayList<Partner>();
		}
		
		for (int i=0; i < this.NUMBER_BACKUP_RATINGS*this.NUMBER_SELECTION_CLASSES; i++) {
			
			if (numInteractionsOrder.size() == 0) break;
			index = getIndex(i, numInteractionsOrder.size(), randomGen);
			nextPartner = numInteractionsOrder.get(index);
			
			int listIndex = i % this.NUMBER_SELECTION_CLASSES;
			
			while(backupPartners[listIndex].contains(nextPartner)) {
				numInteractionsOrder.remove(index);
				if (numInteractionsOrder.size() == 0) break;
				index = getIndex(i, numInteractionsOrder.size(), randomGen);
				nextPartner = numInteractionsOrder.get(index);
			}
			
			backupPartners[listIndex].add(nextPartner);
			numInteractionsOrder.remove(index);			
			
		}
		
		this.ratingPartnersBackup = backupPartners;
		
	}

	private int getIndex(int i, int size, Random rnd) {
		
		// This routine has to be held synchronized according to the ratingPartnersBackup field (NUMBER_SELECTION_CLASSES)
		
		int index = 0;
		switch ((int) i % this.NUMBER_SELECTION_CLASSES) {
		case 0:
			// HIGH INTERACTION
			index = 0;
			break;
		case 1:
			// LOW INTERACTION
			index = size-1;
			break;
		case 2:
			// RANDOM
			index = rnd.nextInt(size);
			break;
		}
		return index;
	}

	/**
	 * Analyze the call- and message-logs
	 * @return A HashMap, where all numbers of calls and messages have been assigned to a partner
	 * (either from the contact list or a new one)
	 */
	private void analyzeCallLogs() {
		Partner partner;

		this.numCalls = 0;
		
		long rightNow = System.currentTimeMillis();
		
		// read in the call and messages logs ******************
		Call newCall = null;
		
		// initialize the partners list
		this.partners = new HashMap<String, Partner>();

		// get the call log
		Cursor callLog = managedQuery(Calls.CONTENT_URI, null, null, null, Calls.NUMBER+" ASC");

		SkimsTrustSurvey.this.numCalls = callLog.getCount();

		// move through the call log and group the calls by partner - that's how we
		// call contacts (contact list entries) that have been communicated with
		for (callLog.moveToFirst();!callLog.isAfterLast();callLog.moveToNext()) {

			// create a new call object
			newCall = new Call();
			newCall.setNumber(callLog.getString(callLog.getColumnIndex(Calls.NUMBER)));
			newCall.setDate(((rightNow - callLog.getLong(callLog.getColumnIndex(Calls.DATE)))/1000)+this.randomDateOffset);
			newCall.setDuration(callLog.getLong(callLog.getColumnIndex(Calls.DURATION)));
			newCall.setType(callLog.getInt(callLog.getColumnIndex(Calls.TYPE)));

			Pair<Partner, Integer> pPair = getPartner(this.partners, newCall.getNumber());

			partner = pPair.getFirst();
			newCall.setNumberType(pPair.getSecond());

			// add call to partner
			partner.addCall(newCall);
		}
		
		callLog.close();
	}
	
	private void analyzeMessageLogs() {
		Partner partner;
		this.numMessages = 0;
		Message newMessage = null;
		
		long rightNow = System.currentTimeMillis();
		
		// get the messages
		String url = "content://sms/"; 
		Uri messageUri = Uri.parse(url); 

		Cursor messageCur = managedQuery(messageUri, null, null, null, null);

		SkimsTrustSurvey.this.numMessages = messageCur.getCount();

		int colAddress = messageCur.getColumnIndex(SMS_COL_ADDRESS);
		int colDate = messageCur.getColumnIndex(SMS_COL_DATE);
		int colType = messageCur.getColumnIndex(SMS_COL_TYPE);
		int colBody = messageCur.getColumnIndex(SMS_COL_BODY);

		// move through the messages
		for (messageCur.moveToFirst();!messageCur.isAfterLast();messageCur.moveToNext()) {

			// create a new message object and populate it
			newMessage = new Message();
			newMessage.setAddress(messageCur.getString(colAddress));
			newMessage.setDate(((rightNow - messageCur.getLong(colDate))/1000)+this.randomDateOffset);
			newMessage.setType(messageCur.getInt(colType));
			newMessage.setLength(messageCur.getString(colBody).length());

			// if this message was not sent it is of no use to us and may not even have an address
			if (newMessage.getType() == Message.TYPE_DRAFT || newMessage.getType() == Message.TYPE_UNDEFINED) {
				this.numMessages--;
				continue;
			}
			
			Pair<Partner, Integer> pPair = getPartner(this.partners, newMessage.getAddress());
			partner = pPair.getFirst();
			newMessage.setNumberType(pPair.getSecond());

			// add message to partner
			partner.addMessage(newMessage);
		}
		
		messageCur.close();
		
	}
	
	private void analyzeContactList() {
		// check out how many contacts are in the contact list
		Cursor contactListCur =  managedQuery(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, null, null, null, null);
		this.numContacts = contactListCur.getCount();
		contactListCur.close();
		
	}

	/**
	 *  Check if number belongs to a partner already in our list if not create a new one and put it in the list.
	 *  This function also returns an integer representing the type of the phone number provided (e.g. work, home...)
	 * @param partners List of partners to choose from
	 * @param number Phone number to use for the query
	 * @return A Pair containing the partner and the type of the phone number
	 */
	private Pair<Partner, Integer> getPartner(Map<String, Partner> partners, String number) {
		// get contact to this number
		String lookupKey = null;
		Partner partner = null;
		
		int phoneNumberType = -1;

		boolean inContactList = false;

		for (Partner thisPartner : partners.values()) {
			if (thisPartner.hasNumber(number)) {
				lookupKey = thisPartner.getLookupKey();
				partner = thisPartner;
				phoneNumberType = thisPartner.getNumberType(number);
				break;
			}
		}
	
		if (partner == null) {

			Uri phoneLookupUri = Uri.withAppendedPath(PhoneLookup.CONTENT_FILTER_URI, Uri.encode(number));
			Cursor phoneLookupCur = managedQuery(
					phoneLookupUri, 
					new String[] {PhoneLookup.LOOKUP_KEY, PhoneLookup.DISPLAY_NAME, PhoneLookup.STARRED, PhoneLookup.SEND_TO_VOICEMAIL, PhoneLookup.TYPE}, 
					null, null, null);
			if (phoneLookupCur != null && phoneLookupCur.moveToFirst()) { 
				lookupKey = phoneLookupCur.getString(phoneLookupCur.getColumnIndex(PhoneLookup.LOOKUP_KEY));
				inContactList = true;
			} else {
				// number is not in contact list
				lookupKey = LOOKUP_KEY_CREATION_PREFIX + number;
			}

			partner = new Partner(lookupKey);
			if (lookupKey.startsWith(LOOKUP_KEY_CREATION_PREFIX, 0)) {
				partner.setDisplayName("Number "+number);
			} else {
				partner.setDisplayName(phoneLookupCur.getString(phoneLookupCur.getColumnIndex(PhoneLookup.DISPLAY_NAME)));
				partner.setStarred(phoneLookupCur.getInt(phoneLookupCur.getColumnIndex(PhoneLookup.STARRED)));
				phoneNumberType =  (phoneLookupCur.getInt(phoneLookupCur.getColumnIndex(PhoneLookup.TYPE)));
			}
			if (!USE_ONLY_CONTACT_LIST_ENTRIES || inContactList) partners.put(lookupKey, partner);
			partner.addNumber(number, phoneNumberType);
			
			phoneLookupCur.close();
		
		}

		partner.setInContactList(inContactList);
				
		return new Pair<Partner, Integer>(partner, phoneNumberType);
	}

	/**
	 * A private Class that organizes the Asynchronous task of collection data
	 * @author trapp
	 *
	 */
	private class LogAnalyzerTask extends AsyncTask<Void, Integer, Void> {

		protected void onPostExecute(Void v) {
			// As soon as logs are analyzed, hide progressBar and show the button to close the window
			ProgressBar progressBar = (ProgressBar) SkimsTrustSurvey.this.startScreen.findViewById(R.id.progressBar1);
			progressBar.setVisibility(View.INVISIBLE);
			TextView tvPleaseWait = (TextView) SkimsTrustSurvey.this.startScreen.findViewById(R.id.tv_please_wait);
			tvPleaseWait.setVisibility(View.INVISIBLE);
			Button button = (Button) SkimsTrustSurvey.this.startScreen.findViewById(R.id.btn_start);
			button.setVisibility(View.VISIBLE);
			//SkimsTrustSurvey.this.startScreen.setCancelable(true);
		}

		protected void onProgressUpdate(Integer... status) {
			ProgressBar progressBar = (ProgressBar) SkimsTrustSurvey.this.startScreen.findViewById(R.id.progressBar1);
			progressBar.setProgress(status[0]);
		}
		
		@Override
		protected Void doInBackground(Void... params) {
			publishProgress(20);
			analyzeCallLogs();
			publishProgress(50);
			analyzeMessageLogs();
			publishProgress(80);
			analyzeContactList();
			publishProgress(90);
			createRatingPartners();
			publishProgress(100);
			return null;
		}


	}

}