package skims.fu.trust.survey;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

public class Sort {
	
	public static final int BY_NUMBER_CALLS = 1;
	public static final int BY_NUMBER_MESSAGES = 2;
	public static final int BY_CUMULATED_CALLS = 3;
	public static final int BY_CUMULATED_MESSAGES = 4;
	public static final int BY_NUMBER_INTERACTIONS = 5;
	public static final int BY_CUMULATED_INTERACTIONS = 6;
	
	
	public static List<Partner> sortPartnerBy(Collection<Partner> originalList, int sortingKey) {
		List<Partner> newList = new LinkedList<Partner>(originalList);
		
		newList = sort(newList, sortingKey);
		
		return newList;
	}
	
	private static LinkedList<Partner> sort(List<Partner> list, int sortingKey) {
		if (list.size() <= 1) return new LinkedList<Partner>(list);
		
		int pivotIdx = list.size() / 2;
		Partner pivot = list.get(pivotIdx);
		
		LinkedList<Partner> less = new LinkedList<Partner>();
		LinkedList<Partner> greater = new LinkedList<Partner>();
		
		for (Partner partner : list) {
			if (partner == pivot) continue;
			if (smaller(partner, pivot, sortingKey)) {
				less.add(partner);
			} else {
				greater.add(partner);
			}
		}
		
		less = sort(less, sortingKey);
		greater = sort(greater, sortingKey);
		
		greater.add(pivot);
		greater.addAll(less);
		return greater;
	
	}
	
	private static boolean smaller(Partner p1, Partner p2, int sortingKey) {
		boolean small = false;
		switch (sortingKey) {
		case Sort.BY_CUMULATED_CALLS:
			small = p1.getCallDuration() < p2.getCallDuration();
			break;
		case Sort.BY_CUMULATED_MESSAGES:
			small = p1.getMessageLength() < p2.getMessageLength();
			break;
		case Sort.BY_NUMBER_CALLS:
			small = p1.getNumCalls() < p2.getNumCalls();
			break;
		case Sort.BY_NUMBER_MESSAGES:
			small = p1.getNumMessages() < p2.getNumMessages();
			break;
		case Sort.BY_NUMBER_INTERACTIONS:
			small = p1.getNumInteractions() < p2.getNumInteractions();
			break;
		case Sort.BY_CUMULATED_INTERACTIONS:
			small = p1.getNumInteractions() < p2.getNumInteractions();
			break;
		}
		return small;
	}
}
